/***************************** Include Files *********************************/

#include "xparameters.h"
#include "xiic.h"
#include "xil_io.h"
#include "xil_printf.h"
#include "xsysmon.h"
#include "main_header.h"     //Function Declarations
#include "p_c_i_bit_test.h"

/************************** Constant Definitions *****************************/

/*
 * The following constants map to the XPAR parameters created in the
 * xparameters.h file. They are defined here such that a user can easily
 * change all the needed parameters in one place.
 */
#define IIC_BASE_ADDRESS	XPAR_IIC_2_BASEADDR

/*
 * The page size determines how much data should be written at a time.
 * The write function should be called with this as a maximum byte count.
 */
#define PAGE_SIZE	2
/*
 * The Starting address in the IIC EEPROM on which this test is performed
 */
#define TEST_START_ADDRESS	0

/**************************** Type Definitions *******************************/
typedef u8 AddressType;

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

unsigned WriteByte_to_Retimer(AddressType Address, u8 *BramBufferPtr,
		u16 ByteCount);

unsigned ReadByte_from_Retimer(AddressType Address, u8 *BramBufferPtr,
		u16 ByteCount);

int IicRetimerConfig();

/************************** Variable Definitions **************************/

int ErrorCount; /* The Error Count */

u8 Writebram_buffer[4]; /* Write BramMemoryfer for writing a page */
u8 Readbram_buffer[4]; /* Read BramMemoryfer for reading a page */
u8 ReadbBufferAll[1 * 4]; /* BramMemoryfer used for reading all the data */

u8 IicAddr; /* Variable for storing Eeprom IIC address */

int Retimer_init() {
	int Status;
	/* Retimer -1 Configuration */
	IicAddr = 0x1A; //0x34

	Status = IicRetimerConfig();
	if (Status != XST_SUCCESS) {
		print("Failed\r\n");
		return XST_FAILURE;
	}

	/* Retimer -2 Configuration */
	IicAddr = 0x18; //0x30

	Status = IicRetimerConfig();
	if (Status != XST_SUCCESS) {
		print("Failed\r\n");
		return XST_FAILURE;
	}

	/* Retimer -3 Configuration */
	IicAddr = 0x1B; //0x36

	Status = IicRetimerConfig();
	if (Status != XST_SUCCESS) {
		print("Failed\r\n");
		return XST_FAILURE;
	}

	/* Retimer -4 Configuration */
	IicAddr = 0x19; //0x32

	Status = IicRetimerConfig();
	if (Status != XST_SUCCESS) {
		print("Failed\r\n");
		return XST_FAILURE;
	}

	return 0;
}

int IicRetimerConfig() {
	int Status;
	u8 BytesWritten;
	u8 clear_curr_val;
	u8 clear_new_val;
	u8 combined;

	u8 Reg_Mask;
	u8 Reg_Val;

	//Sanity Default Register Reads
	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x01, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x04, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x05, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x07, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	//Configuration of VCO Frequency Tuning
	Writebram_buffer[0] = 0x0e;
	BytesWritten = WriteByte_to_Retimer(0xff, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x36, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0x30;
	Reg_Val = 0x30;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x36, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x2f, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0xFF;
	Reg_Val = 0x66;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x2f, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x60, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0xFF;
	Reg_Val = 0x80;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x60, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x61, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0xFF;
	Reg_Val = 0xaa;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x61, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x62, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0xFF;
	Reg_Val = 0x80;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x62, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x63, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0xFF;
	Reg_Val = 0xaa;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x63, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x64, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0xFF;
	Reg_Val = 0xFF;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x64, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x0A, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0x0C;
	Reg_Val = 0x0C;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x0A, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x0A, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x0A, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Reg_Mask = 0x0C;
	Reg_Val = 0x00;

	clear_curr_val = Readbram_buffer[0] & ~Reg_Mask;
	clear_new_val = Reg_Val & Reg_Mask;
	combined = clear_new_val | clear_curr_val;

	Writebram_buffer[0] = combined;
	BytesWritten = WriteByte_to_Retimer(0x0A, Writebram_buffer, 2);
	if (BytesWritten != PAGE_SIZE) {
		return XST_FAILURE;
	}

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x0A, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0xFF, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x36, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x2f, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x60, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x61, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x62, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x63, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	Readbram_buffer[0] = 0;
	Status = ReadByte_from_Retimer(0x64, Readbram_buffer, 1);
	if (Status != XST_SUCCESS)
		return XST_FAILURE;

	MB_Sleep(1);

	return XST_SUCCESS;
}
/*****************************************************************************/
/**
 * This function writes a BramMemoryfer of bytes to the LM75B IIC Slave.
 *
 * @param	Address contains the address in the EEPROM to write to.
 * @param	BramBufferPtr contains the address of the data to write.
 * @param	ByteCount contains the number of bytes in the BramMemoryfer to be written.
 *		Note that this should not exceed the page size of the EEPROM as
 *		noted by the constant PAGE_SIZE.
 *
 * @return	The number of bytes written, a value less than that which was
 *		specified as an input indicates an error.
 *
 * @note		None.
 *
 ****************************************************************************/
unsigned WriteByte_to_Retimer(AddressType Address, u8 *BramBufferPtr,
		u16 ByteCount) {
	volatile unsigned SentByteCount;
	volatile unsigned AckByteCount;
	u8 Writebram_buffer[sizeof(Address) + PAGE_SIZE];
	int Index;
	u32 CntlReg;

	/*
	 * A temporary write BramMemoryfer must be used which contains both the address
	 * and the data to be written, put the address in first based upon the
	 * size of the address for the EEPROM.
	 */
	if (sizeof(AddressType) == 2) {
		Writebram_buffer[0] = (u8) (Address >> 8);
		Writebram_buffer[1] = (u8) (Address);
	} else if (sizeof(AddressType) == 1) {
		Writebram_buffer[0] = (u8) (Address);
		IicAddr |= (TEST_START_ADDRESS >> 8) & 0x7;
	}

	/*
	 * Put the data in the write BramMemoryfer following the address.
	 */

	for (Index = 0; Index < ByteCount; Index++) {
		Writebram_buffer[sizeof(Address) + Index] = BramBufferPtr[Index];
	}

	/*
	 * Set the address register to the specified address by writing
	 * the address to the device, this must be tried until it succeeds
	 * because a previous write to the device could be pending and it
	 * will not ack until that write is complete.
	 */
	do {
		SentByteCount = XIic_Send(IIC_BASE_ADDRESS, IicAddr, (u8 *) &Address,
				sizeof(Address),
				XIIC_STOP);
		if (SentByteCount != sizeof(Address)) {

			/* Send is aborted so reset Tx FIFO */
			CntlReg = XIic_ReadReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET);
			XIic_WriteReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET,
					CntlReg | XIIC_CR_TX_FIFO_RESET_MASK);
			XIic_WriteReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET,
					XIIC_CR_ENABLE_DEVICE_MASK);
		}

	} while (SentByteCount != sizeof(Address));

	/*
	 * Write a page of data at the specified address to the EEPROM.
	 */
	SentByteCount = XIic_Send(IIC_BASE_ADDRESS, IicAddr, Writebram_buffer,
			sizeof(Address) + PAGE_SIZE,
			XIIC_STOP);

	/*
	 * Wait for the write to be complete by trying to do a write and
	 * the device will not ack if the write is still active.
	 */
	do {
		AckByteCount = XIic_Send(IIC_BASE_ADDRESS, IicAddr, (u8 *) &Address,
				sizeof(Address),
				XIIC_STOP);
		if (AckByteCount != sizeof(Address)) {

			/* Send is aborted so reset Tx FIFO */
			CntlReg = XIic_ReadReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET);
			XIic_WriteReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET,
					CntlReg | XIIC_CR_TX_FIFO_RESET_MASK);
			XIic_WriteReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET,
					XIIC_CR_ENABLE_DEVICE_MASK);
		}

	} while (AckByteCount != sizeof(Address));

	/*
	 * Return the number of bytes written to the EEPROM
	 */
	return SentByteCount - sizeof(Address);
}

/*****************************************************************************/
/**
 * This function reads a number of bytes from LM75B IIC slave into a
 * specified BramMemoryfer.
 *
 * @param	Address contains the address in the EEPROM to read from.
 * @param	BramBufferPtr contains the address of the data BramMemoryfer to be filled.
 * @param	ByteCount contains the number of bytes in the BramMemoryfer to be read.
 *		This value is not constrained by the page size of the device
 *		such that up to 64K may be read in one call.
 *
 * @return	The number of bytes read. A value less than the specified input
 *		value indicates an error.
 *
 * @note		None.
 *
 ****************************************************************************/
unsigned ReadByte_from_Retimer(AddressType Address, u8 *BramBufferPtr,
		u16 ByteCount) {
	volatile unsigned ReceivedByteCount;
	u16 StatusReg;
	u32 CntlReg;
	/*
	 * Set the address register to the specified address by writing
	 * the address to the device, this must be tried until it succeeds
	 * because a previous write to the device could be pending and it
	 * will not ack until that write is complete.
	 */
	do {
		StatusReg = XIic_ReadReg(IIC_BASE_ADDRESS, XIIC_SR_REG_OFFSET);
		if (!(StatusReg & XIIC_SR_BUS_BUSY_MASK)) {
			ReceivedByteCount = XIic_Send(IIC_BASE_ADDRESS, IicAddr,
					(u8 *) &Address, sizeof(Address),
					XIIC_STOP);

			if (ReceivedByteCount != sizeof(Address)) {

				/* Send is aborted so reset Tx FIFO */
				CntlReg = XIic_ReadReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET);
				XIic_WriteReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET,
						CntlReg | XIIC_CR_TX_FIFO_RESET_MASK);
				XIic_WriteReg(IIC_BASE_ADDRESS, XIIC_CR_REG_OFFSET,
						XIIC_CR_ENABLE_DEVICE_MASK);
			}
		}

	} while (ReceivedByteCount != sizeof(Address));

	/*
	 * Read the number of bytes at the specified address from the EEPROM.
	 */
	ReceivedByteCount = XIic_Recv(IIC_BASE_ADDRESS, IicAddr, BramBufferPtr,
			ByteCount, XIIC_STOP);



	return XST_SUCCESS;
}
